/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.organdonor.backgroundjob.health;

import java.util.Comparator;
import java.util.List;
import java.util.Objects;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * The MIT License
 * <p>
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * <p>
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * <p>
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 * <p>
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * <p>
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 * <p>
 * Created by MacAni on 26/06/2017 11.37.
 */

public class HealthReportGenerator {
    public static String generateResult(List<HealthIndicator> healthIndicators) {
        String content = healthIndicators.stream()
                .map(indicator -> indicator.getHealthInfo().getText()).collect(Collectors.joining(", "));
        return String.format("{ %s }", content);
    }

    public static Integer generateStatus(List<HealthIndicator> healthIndicators) {
        return healthIndicators.stream()
                .map(indicator -> indicator.getHealthInfo().getStatus())
                .max(Comparator.comparing(Function.identity()))
                .orElse(200);
    }

    public static List<String> generateStatusResult(List<StatusIndicator> statusIndicators) {
        List<String> content = statusIndicators.stream()
                .map(status -> status.getStatusInfo().getTextAsKeyValue())
                .filter(Objects::nonNull)
                .collect(Collectors.toList());
        return content;
    }

    public static Integer generateStatusStatus(List<StatusIndicator> statusIndicators) {
        return statusIndicators.stream()
                .map(indicator -> indicator.getStatusInfo().getStatus())
                .max(Comparator.comparing(Function.identity()))
                .orElse(200);
    }


    public static List<String> generateAlarmResult(List<AlarmIndicator> alarmIndicators) {
        List<String> content = alarmIndicators.stream()
                .map(alarm -> alarm.getAlarmInfo().getTextAsFullText())
                .filter(Objects::nonNull)
                .collect(Collectors.toList());
        return content;

    }

    public static Integer generateAlarmStatus(List<AlarmIndicator> alarmIndicators) {
        return alarmIndicators.stream()
                .map(indicator -> indicator.getAlarmInfo().getStatus())
                .max(Comparator.comparing(Function.identity()))
                .orElse(200);
    }

}
