/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.organdonor.backgroundjob.util;

import java.time.LocalDate;

public class DateStream implements Iterable<LocalDate> {
    private final LocalDate fromInclusive;
    private final LocalDate toExclusive;

    public DateStream(LocalDate fromInclusive, LocalDate toExclusive) {
        this.fromInclusive = fromInclusive;
        this.toExclusive = toExclusive;
    }

    /**
     * Returns a stream of all dates from 2000-01-01 to 2000-12-31
     * @return
     */
    public static DateStream getLeapYearStream(){
        //Vi bruger år 2000 som vi ved er skudår!
        LocalDate fromInclusive = LocalDate.of(2000, 1, 1);
        return new DateStream(fromInclusive, fromInclusive.plusYears(1));
    }

    @Override
    public java.util.Iterator<LocalDate> iterator() {
        return new java.util.Iterator<LocalDate>() {
            private LocalDate current = fromInclusive;

            @Override
            public boolean hasNext() {
                return current.isBefore(toExclusive);
            }

            @Override
            public LocalDate next() {
                if(!hasNext()) {
                    throw new java.util.NoSuchElementException();
                }
                LocalDate next = current;
                current = current.plusDays(1);
                return next;
            }
        };
    }
}
