/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.organdonor.dao;

import dk.sundhedsdatastyrelsen.organdonor.backgroundjob.dao.AlivenessDao;
import dk.sundhedsdatastyrelsen.organdonor.dao.exceptions.DAOException;
import dk.sundhedsdatastyrelsen.organdonor.domain.OrganDonorVO;
import org.springframework.dao.DataAccessException;

import java.time.Instant;
import java.util.LinkedList;
import java.util.List;

public interface OrganDonorDAO extends AlivenessDao {
    Integer createOrganDonorRegistration(OrganDonorVO organDonorVO, Instant nowTimestamp) throws DAOException;
    OrganDonorVO getOrganDonorRegistration(String personIdentifier, Instant nowTimestamp) throws DAOException;
    boolean existsOrganDonorRegistrationRecords(String personIdentifier) throws DAOException;
    void updateOrganDonorRegistration(Integer organDonorPID, OrganDonorVO organDonorVO, Instant nowTimeStamp) throws DAOException;
    void deleteOrganDonorRegistration(Integer pidToUpdate, Instant nowTimestamp) throws DAOException;
    @Deprecated
    void deleteOrganDonorRegistrationRecords(String jobStartTime) throws DAOException;
    @Deprecated
    LinkedList<String> getallPersonsFromODR() throws DAOException;

    List<String> getPersonIdBatch(String personIdPrefix) throws DataAccessException;
    List<String> getValidPersonIdBatch(String personIdPrefix) throws DataAccessException;
    int deleteEntriesByPersonIdBatch(List<String> personIdBatch) throws DataAccessException;
    int updateEntriesByPersonIdBatch(List<String> personIdBatch, Instant nowTimestamp) throws DataAccessException;
}
