/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.organdonor.personinformation;

import org.apache.log4j.Logger;
import org.openapitools.model.Custody;

import java.time.Instant;
import java.util.List;

public class PersonInformationImpl implements PersonInformation {

	private static final Logger LOGGER = Logger.getLogger(PersonInformationImpl.class);

	private final PersonInformationClient personInformationClient;

	private static final Logger logger = Logger.getLogger(PersonInformationImpl.class);

    public PersonInformationImpl(PersonInformationClient personInformationClient) {
		this.personInformationClient = personInformationClient;
	}

	@Override
	public Custody getCustody(String patientId) {
		
		try {
			return personInformationClient.getCustody(patientId);
		} catch (PersonInformationException e) {
			LOGGER.error("Kaldet til PersonInformation fejlede", e);
			throw e;
		}
	}

	@Override
	public String getName(String patientId) {
		try {
			return personInformationClient.getName(patientId);
		} catch (PersonInformationException e) {
			LOGGER.error("Kaldet til PersonInformation fejlede", e);
			throw e;
		}
	}

	@Override
	public int getAge(String patientId) {
		try {
			return personInformationClient.getAge(patientId);
		} catch (PersonInformationException e) {
			LOGGER.error("Kaldet til PersonInformation fejlede", e);
			throw e;
		}
	}

	@Override
	public List<String> getDeceased(List<String> cprList, Instant deceasedDate) {
		try {
			return personInformationClient.getDeceased(cprList, deceasedDate);
		} catch (PersonInformationException e) {
			LOGGER.error("Kaldet til PersonInformation fejlede", e);
			throw e;
		}
	}

	@Override
	public List<String> filterByStatusCode(List<String> cprList, Instant emigratedSinceDate, String statusCode) {
		try {
			return personInformationClient.doFilterByStatusCode(cprList, emigratedSinceDate, statusCode);
		} catch (PersonInformationException e) {
			LOGGER.error("Kaldet til PersonInformation fejlede", e);
			throw e;
		}
	}
}

