/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.organdonor.personinformation;

import java.time.Duration;
import java.time.OffsetDateTime;
import java.time.ZoneId;
import java.util.concurrent.ConcurrentLinkedQueue;

public class ServiceErrorCallback {
    private final Duration errorCountDuration;
    private final ConcurrentLinkedQueue<OffsetDateTime> errorTimestamps = new ConcurrentLinkedQueue<>();
    private final ZoneId zoneId;

    public ServiceErrorCallback(Duration errorCountDuration, ZoneId zoneId) {
        this.errorCountDuration = errorCountDuration;
        this.zoneId = zoneId;
    }

    public void registerServiceError() {
        errorTimestamps.add(OffsetDateTime.now(zoneId));
    }

    public void registerServiceSuccess() {
        errorTimestamps.clear();
    }

    public int getErrorCount() {
        OffsetDateTime from = OffsetDateTime.now(zoneId).minus(errorCountDuration);
        errorTimestamps.removeIf(timestamp -> timestamp.isBefore(from));
        return errorTimestamps.size();
    }
}
