/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.organdonor.backgroundjob.util;

import org.junit.Assert;
import org.junit.Test;

import java.util.Arrays;
import java.util.Collection;
import java.util.List;

public class BatchUtilTest {
    @Test
    public void testBatchUtilEmptyList() {
        Collection<String> input = Arrays.asList();
        Collection<List<String>> batches = BatchUtil.createBatches(input, 5);
        Assert.assertEquals(0, batches.size());
        List[] batchArray = batches.toArray(new List[0]);
    }

    @Test
    public void testBatchUtilBalancedNotShuffled() {
        Collection<String> input = Arrays.asList("A", "B", "C", "D", "E", "F", "G", "H", "I", "J");
        Collection<List<String>> batches = BatchUtil.createBatches(input, 5);
        Assert.assertEquals(2, batches.size());
        List[] batchArray = batches.toArray(new List[0]);
        Assert.assertEquals(5, batchArray[0].size());
        Assert.assertEquals(Arrays.asList("A", "B", "C", "D", "E"), batchArray[0]);
        Assert.assertEquals(5, batchArray[1].size());
        Assert.assertEquals(Arrays.asList("F", "G", "H", "I", "J"), batchArray[1]);
    }

    @Test
    public void testBatchUtilNotBalancedNotShuffled() {
        Collection<String> input = Arrays.asList("A", "B", "C", "D", "E", "F", "G", "H", "I", "J");
        Collection<List<String>> batches = BatchUtil.createBatches(input, 3);
        Assert.assertEquals(4, batches.size());
        List[] batchArray = batches.toArray(new List[0]);
        Assert.assertEquals(3, batchArray[0].size());
        Assert.assertEquals(Arrays.asList("A", "B", "C"), batchArray[0]);
        Assert.assertEquals(3, batchArray[1].size());
        Assert.assertEquals(Arrays.asList("D", "E", "F"), batchArray[1]);
        Assert.assertEquals(3, batchArray[2].size());
        Assert.assertEquals(Arrays.asList("G", "H", "I"), batchArray[2]);
        Assert.assertEquals(1, batchArray[3].size());
        Assert.assertEquals(Arrays.asList("J"), batchArray[3]);
    }

    @Test
    public void testBatchUtilShuffledBalanced() {
        Collection<String> input = Arrays.asList("A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T");
        Collection<List<String>> batches = BatchUtil.createShuffledBatches(input, 5);
        Assert.assertEquals(4, batches.size());
        List[] batchArray = batches.toArray(new List[0]);
        Assert.assertEquals(5, batchArray[0].size());
        Assert.assertNotEquals(Arrays.asList("A", "B", "C", "D", "E"), batchArray[0]);
        Assert.assertEquals(5, batchArray[1].size());
        Assert.assertNotEquals(Arrays.asList("F", "G", "H", "I", "J"), batchArray[1]);
        Assert.assertEquals(5, batchArray[2].size());
        Assert.assertNotEquals(Arrays.asList("K", "L", "M", "N", "O"), batchArray[2]);
        Assert.assertEquals(5, batchArray[3].size());
        Assert.assertNotEquals(Arrays.asList("P", "Q", "R", "S", "T"), batchArray[3]);
    }

    @Test
    public void testBatchUtilShuffledNotBalanced() {
        Collection<String> input = Arrays.asList("A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T");
        Collection<List<String>> batches = BatchUtil.createShuffledBatches(input, 6);
        Assert.assertEquals(4, batches.size());
        List[] batchArray = batches.toArray(new List[0]);
        Assert.assertEquals(6, batchArray[0].size());
        Assert.assertNotEquals(Arrays.asList("A", "B", "C", "D", "E", "F"), batchArray[0]);
        Assert.assertEquals(6, batchArray[1].size());
        Assert.assertNotEquals(Arrays.asList("G", "H", "I", "J", "K", "L"), batchArray[1]);
        Assert.assertEquals(6, batchArray[2].size());
        Assert.assertNotEquals(Arrays.asList("M", "N", "O", "P", "Q", "R"), batchArray[2]);
        Assert.assertEquals(2, batchArray[3].size());
        Assert.assertNotEquals(Arrays.asList("S", "T"), batchArray[3]);
    }
}
