/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.organdonor.cleanup.config;

import dk.sundhedsdatastyrelsen.organdonor.backgroundjob.service.StackedOperationsService;
import dk.sundhedsdatastyrelsen.organdonor.backgroundjob.util.DateTimeProvider;
import dk.sundhedsdatastyrelsen.organdonor.cleanup.factory.CleanupOperationsFactory;
import dk.sundhedsdatastyrelsen.organdonor.cleanup.service.CleanupService;
import dk.sundhedsdatastyrelsen.organdonor.cleanup.service.DBHealthIndicator;
import dk.sundhedsdatastyrelsen.organdonor.cleanup.stack.deceased.OrganDonorDeceasedCleanupSupplier;
import dk.sundhedsdatastyrelsen.organdonor.cleanup.stack.emigrated.OrganDonorEmigratedCleanupSupplier;
import dk.sundhedsdatastyrelsen.organdonor.dao.OrganDonorDAO;
import dk.sundhedsdatastyrelsen.organdonor.service.PersonInformationService;
import dk.sundhedsdatastyrelsen.organdonor.util.Properties;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Import;

import java.time.Duration;

@Configuration
@Import({CleanupPropertiesConfiguration.class, PersonInformationConfiguration.class})
public class CleanupCoreConfiguration {

    @Bean
    public CleanupService cleanupService(@Autowired OrganDonorDAO dao) {
        return new CleanupService(dao);
    }

    @Bean
    public DBHealthIndicator dbHealthIndicator(@Autowired OrganDonorDAO dao, @Autowired Properties properties) {
        return new DBHealthIndicator(dao, properties.getDataSourceOdrJNDIName());
    }

    @Bean
    public CleanupOperationsFactory cleanupOperationsFactory(CleanupProperties cleanupProperties, CleanupService cleanupService, PersonInformationService personInformationService, DateTimeProvider dateTimeProvider) {
        return new CleanupOperationsFactory(cleanupProperties, cleanupService, personInformationService, dateTimeProvider);
    }

    @Bean
    public Duration desiredCleanupDeceasedExecutionDuration(CleanupProperties cleanupProperties) {
        return Duration.parse(cleanupProperties.getDeletionDeceasedDesiredExecutionDuration());
    }

    @Bean
    public Duration desiredCleanupEmigratedExecutionDuration(CleanupProperties cleanupProperties) {
        return Duration.parse(cleanupProperties.getDeletionEmigratedDesiredExecutionDuration());
    }

    @Bean
    public DateTimeProvider dateTimeProvider() {
        return new DateTimeProvider();
    }

    @Bean
    public StackedOperationsService organDonorDeceasedCleanupService(CleanupOperationsFactory cleanupOperationsFactory,
                                                                     Duration desiredCleanupDeceasedExecutionDuration,
                                                                     DateTimeProvider dateTimeProvider) {
        OrganDonorDeceasedCleanupSupplier supplier = new OrganDonorDeceasedCleanupSupplier(cleanupOperationsFactory);
        return new StackedOperationsService(supplier, desiredCleanupDeceasedExecutionDuration, dateTimeProvider).withServiceName("Organdonor Deceased Cleanup");
    }

    @Bean
    public StackedOperationsService organDonorEmigratedCleanupService(CleanupOperationsFactory cleanupOperationsFactory,
                                                                     Duration desiredCleanupEmigratedExecutionDuration,
                                                                     DateTimeProvider dateTimeProvider) {
        OrganDonorEmigratedCleanupSupplier supplier = new OrganDonorEmigratedCleanupSupplier(cleanupOperationsFactory);
        return new StackedOperationsService(supplier, desiredCleanupEmigratedExecutionDuration, dateTimeProvider).withServiceName("Organdonor Emigrated Cleanup");
    }
}
