/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.organdonor.cleanup.config;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.PropertySource;
import org.springframework.stereotype.Component;

import java.time.Period;

@Component
@PropertySource("classpath:cleanup.properties")
public class CleanupPropertiesConfiguration {
    /**
     * The desired execution duration for the deceased cleanup job.
     */
    @Value("${deletion.desired.execution.duration.deceased:PT20S}")
    private String deletionDeceasedDesiredExecutionDuration;

    /**
     * The desired execution duration for the emigrated cleanup job.
     */
    @Value("${deletion.desired.execution.duration.emigrated:PT20S}")
    private String deletionEmigratedDesiredExecutionDuration;

    /**
     * The number of ids to delete in a single update (Used by deceased).
     */
    @Value("${deletion.batchsize.deceased:1000}")
    private int deletionDeceasedBatchSize;

    /**
     * The number of ids to update in a single update (Used by migrated).
     */
    @Value("${deletion.batchsize.emigrated:1000}")
    private int deletionEmigratedBatchSize;

    /**
     * The number of ids allowed when calling deceased function on PersonInformation service.
     */
    @Value("${personinformation.batchsize.deceased:100}")
    private int personinformationDeceasedBatchSize;

    /**
     * The number of ids allowed when calling filterByStatuscode("80") function on PersonInformation service.
     * Dette kald er et POST kald, så vi kan have flere id'er med i kaldet.
     */
    @Value("${personinformation.batchsize.emigrated:1000}")
    private int personinformationEmigratedBatchSize;

    /**
     * The period a person have to be deceased before it is safe to delete from db
     */
    @Value("${deletion.save.deceased:P60D}")
    private String deletionSaveDeceased;

    @Bean
    public CleanupProperties cleanupProperties() {
        return new CleanupProperties() {
            @Override
            public String getDeletionDeceasedDesiredExecutionDuration() {
                return deletionDeceasedDesiredExecutionDuration;
            }

            @Override
            public int getDeletionDeceasedBatchSize() {
                return deletionDeceasedBatchSize;
            }

            @Override
            public int getPersoninformationDeceasedBatchSize() {
                return personinformationDeceasedBatchSize;
            }

            @Override
            public Period getDeceasedPeriod() {
                return Period.parse(deletionSaveDeceased);
            }

            @Override
            public String getDeletionEmigratedDesiredExecutionDuration() {
                return deletionEmigratedDesiredExecutionDuration;
            }

            @Override
            public int getDeletionEmigratedBatchSize() {
                return deletionEmigratedBatchSize;
            }

            @Override
            public int getPersoninformationEmigratedBatchSize() {
                return personinformationEmigratedBatchSize;
            }
        };
    }
}
