/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.organdonor.cleanup.config;

import dk.sundhedsdatastyrelsen.organdonor.cleanup.service.PersonInformationHealthIndicator;
import dk.sundhedsdatastyrelsen.organdonor.personinformation.PersonInformation;
import dk.sundhedsdatastyrelsen.organdonor.personinformation.PersonInformationClient;
import dk.sundhedsdatastyrelsen.organdonor.personinformation.PersonInformationImpl;
import dk.sundhedsdatastyrelsen.organdonor.personinformation.PoolingHttpClientFactory;
import dk.sundhedsdatastyrelsen.organdonor.service.PersonInformationService;
import org.apache.http.impl.client.CloseableHttpClient;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.PropertySource;
import org.springframework.stereotype.Component;

import java.time.Duration;
import java.time.ZoneId;

@Component
@PropertySource("classpath:cleanup.properties")
public class PersonInformationConfiguration {
    @Value("${personinformation.url}")
    private String personinformationUrl;

    @Value("${personinformation.maxTotalConnections:200}")
    private int personinformationMaxTotalConnections;

    @Value("${personinformationDefaultMaxConnectionsPerRoute:20}")
    private int personinformationDefaultMaxConnectionsPerRoute;

    @Value("${personinformation.error.tolerance:0}")
    private int personInformationErrorTolerance;

    @Value("${personinformation.errorcount.duration:PT10M}")
    private String personInformationErrorCountDuration;

    @Bean
    public PoolingHttpClientFactory poolingHttpClientFactory() {
        return new PoolingHttpClientFactory(personinformationMaxTotalConnections, personinformationDefaultMaxConnectionsPerRoute);
    }

    @Bean
    public ZoneId zoneId() {
        return ZoneId.of("Europe/Copenhagen");
    }

    @Bean
    public PersonInformationClient personInformationClient(PoolingHttpClientFactory poolingHttpClientFactory, ZoneId zoneId) {
        CloseableHttpClient httpClient = poolingHttpClientFactory.getClient();
        return new PersonInformationClient(httpClient, personinformationUrl, Duration.parse(personInformationErrorCountDuration), zoneId, personInformationErrorTolerance);
    }

    @Bean
    public PersonInformation personInformation(PersonInformationClient personInformationClient) {
        return new PersonInformationImpl(personInformationClient);
    }

    @Bean
    public PersonInformationService personInformationService(PersonInformation personInformation) {
        return new PersonInformationService(personInformation);
    }

    @Bean
    public PersonInformationHealthIndicator personInformationHealthIndicator(PersonInformationClient client) {
        return new PersonInformationHealthIndicator(client);
    }
}
