/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.organdonor.cleanup.service;

import dk.sundhedsdatastyrelsen.organdonor.dao.OrganDonorDAO;
import org.apache.log4j.Logger;
import org.springframework.dao.DataAccessException;

import java.time.Instant;
import java.util.List;

public class CleanupService {
    private static final Logger log = Logger.getLogger(CleanupService.class);

    private final OrganDonorDAO dao;

    public CleanupService(OrganDonorDAO dao) {
        this.dao = dao;
    }

    public List<String> getPersonIdBatchByPersonIdPrefix(String personIdPrefix) {
        try {
            log.trace("Getting person id batch by prefix: " + personIdPrefix);
            return dao.getPersonIdBatch(personIdPrefix);
        } catch (DataAccessException e) {
            throw new CleanupException(ErrorMessages.cleanupCouldNotAccessDatabase(), e);
        }
    }

    public List<String> getValidPersonIdBatchByPersonIdPrefix(String personIdPrefix) {
        try {
            log.trace("Getting person id batch by prefix: " + personIdPrefix);
            return dao.getValidPersonIdBatch(personIdPrefix);
        } catch (DataAccessException e) {
            throw new CleanupException(ErrorMessages.cleanupCouldNotAccessDatabase(), e);
        }
    }

    public int deleteEntriesByPersonIdBatch(List<String> personIdBatch) {
        try {
            log.trace("Deleting log "+personIdBatch.size()+" entries by person id batch.");
            return dao.deleteEntriesByPersonIdBatch(personIdBatch);
        } catch (DataAccessException e) {
            throw new CleanupException(ErrorMessages.cleanupCouldNotAccessDatabase(), e);
        }
    }

    public int updateEntriesByPersonIdBatch(List<String> personIdBatch, Instant nowTimestamp) {
        try {
            log.trace("Updating log "+personIdBatch.size()+" entries by person id batch.");
            return dao.updateEntriesByPersonIdBatch(personIdBatch, nowTimestamp);
        } catch (DataAccessException e) {
            throw new CleanupException(ErrorMessages.cleanupCouldNotAccessDatabase(), e);
        }
    }
}
