/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.organdonor.cleanup.service;

import dk.sundhedsdatastyrelsen.organdonor.backgroundjob.dao.AlivenessDao;
import dk.sundhedsdatastyrelsen.organdonor.backgroundjob.health.AlarmIndicator;
import dk.sundhedsdatastyrelsen.organdonor.backgroundjob.health.HealthIndication;
import dk.sundhedsdatastyrelsen.organdonor.backgroundjob.health.HealthIndicator;
import dk.sundhedsdatastyrelsen.organdonor.backgroundjob.health.KeyValuePair;
import dk.sundhedsdatastyrelsen.organdonor.backgroundjob.health.StatusIndicator;
import org.apache.http.HttpStatus;

public class DBHealthIndicator implements HealthIndicator, StatusIndicator, AlarmIndicator {
    private final AlivenessDao alivenessDao;
    private final String dataSourceName;

    public DBHealthIndicator(AlivenessDao dao, String dataSourceName) {
       this.alivenessDao = dao;
       this.dataSourceName = dataSourceName;
    }

    @Override
    public HealthIndication getHealthInfo() {
        return alivenessDao.isAlive() ? new HealthIndication(HttpStatus.SC_OK, new KeyValuePair("Database", "OK"))
                : new HealthIndication(HttpStatus.SC_SERVICE_UNAVAILABLE, new KeyValuePair("Database", "Unavailable"));
    }

    @Override
    public HealthIndication getStatusInfo() {
        return alivenessDao.isAlive() ? new HealthIndication(HttpStatus.SC_OK, new KeyValuePair("Database", "OK"))
                : new HealthIndication(HttpStatus.SC_INTERNAL_SERVER_ERROR, new KeyValuePair("Database", "Unavailable"));
    }
    @Override
    public HealthIndication getAlarmInfo() {
        return alivenessDao.isAlive() ? new HealthIndication(HttpStatus.SC_OK, null)
                : new HealthIndication(HttpStatus.SC_INTERNAL_SERVER_ERROR, new KeyValuePair(null, ErrorMessages.dbHealthIndicatorAlarm(dataSourceName)));
    }
}
