/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.organdonor.cleanup.stack.emigrated;

import dk.sundhedsdatastyrelsen.organdonor.backgroundjob.stack.StackOperation;
import dk.sundhedsdatastyrelsen.organdonor.backgroundjob.util.DateTimeProvider;
import dk.sundhedsdatastyrelsen.organdonor.cleanup.service.CleanupException;
import dk.sundhedsdatastyrelsen.organdonor.cleanup.service.CleanupService;
import dk.sundhedsdatastyrelsen.organdonor.cleanup.service.ErrorMessages;
import dk.sundhedsdatastyrelsen.organdonor.cleanup.stack.AbstractCleanupOperation;

import java.time.Duration;
import java.time.ZonedDateTime;
import java.util.Collections;
import java.util.List;

public class CPREmigratedCleanupOperation extends AbstractCleanupOperation {
    private static final String OPERATION_NAME = "OrganDonor Migrated Cleanup";

    private final String prefix;
    private final List<String> personIdsForUpdate;

    public CPREmigratedCleanupOperation(CleanupService cleanupService, DateTimeProvider dateTimeProvider, String prefix, List<String> personIdsForUpdate) {
        super(cleanupService, dateTimeProvider);
        this.prefix = prefix;
        this.personIdsForUpdate = personIdsForUpdate;
        log.debug("CPRMigratedCleanupOperation created with " + personIdsForUpdate.size() + " records");
    }

    @Override
    public String getIdentity() {
        return getClass().getSimpleName()+"{prefix=" + prefix + ", size=" + personIdsForUpdate.size() + "}";
    }

    @Override
    public List<StackOperation> run() {
        log.trace(String.format("%s: Starting batch updating of %d entries.", OPERATION_NAME, personIdsForUpdate.size()));
        ZonedDateTime startTime = dateTimeProvider.now();

        try {
            // Angiv tildato for alle entries for personer, der skal opdateres
            cleanupService.updateEntriesByPersonIdBatch(personIdsForUpdate, dateTimeProvider.now().toInstant());
        } catch (CleanupException e) {
            throw e;
        } catch (Exception e) { // catch all, fordi vi gerne vil have en god tekst til alarm endpointet.
            throw new CleanupException(ErrorMessages.cleanupDeleteFailed(personIdsForUpdate.size()), e);
        }
        log.info(String.format("%s: Batch update finished! updated=%d Time spent: %d seconds.", OPERATION_NAME, personIdsForUpdate.size(), Duration.between(startTime, dateTimeProvider.now()).getSeconds()));

        return Collections.emptyList();
    }
}
