/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.organdonor.cleanup.stack.emigrated;

import dk.sundhedsdatastyrelsen.organdonor.backgroundjob.stack.StackOperation;
import dk.sundhedsdatastyrelsen.organdonor.backgroundjob.util.DateStream;
import dk.sundhedsdatastyrelsen.organdonor.cleanup.factory.CleanupOperationsFactory;
import dk.sundhedsdatastyrelsen.organdonor.cleanup.stack.AbstractCleanupSupplier;

import java.time.format.DateTimeFormatter;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.StreamSupport;

public class OrganDonorEmigratedCleanupSupplier extends AbstractCleanupSupplier {
    /**
     * Prefixes for the cleanup operations. Contains all possible prefixes ddMM in a year.
     */
    public static final List<String> PREFIX_ddMM = StreamSupport.stream(DateStream.getLeapYearStream().spliterator(), false)
            .map(DateTimeFormatter.ofPattern("ddMM")::format)
            .collect(Collectors.collectingAndThen(Collectors.toList(), Collections::unmodifiableList));

    public OrganDonorEmigratedCleanupSupplier(CleanupOperationsFactory cleanupOperationsFactory) {
        super(cleanupOperationsFactory);
        log.debug("OrganDonorEmigratedCleanupSupplier created");
    }

    @Override
    public List<StackOperation> run() {
        log.debug("OrganDonorEmigratedCleanupSupplier executing");
        return PREFIX_ddMM.stream().map(cleanupOperationsFactory::createCPRPrefixEmigratedCleanupSupplier).collect(Collectors.toList());
    }
}
