/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.organdonor.cleanup.stack.deceased;

import dk.sundhedsdatastyrelsen.organdonor.backgroundjob.stack.StackOperation;
import dk.sundhedsdatastyrelsen.organdonor.backgroundjob.util.DateTimeProvider;
import dk.sundhedsdatastyrelsen.organdonor.cleanup.config.CleanupProperties;
import dk.sundhedsdatastyrelsen.organdonor.cleanup.factory.CleanupOperationsFactory;
import dk.sundhedsdatastyrelsen.organdonor.cleanup.service.CleanupService;
import dk.sundhedsdatastyrelsen.organdonor.service.PersonInformationService;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.junit.MockitoJUnitRunner;

import java.util.List;

import static org.junit.Assert.assertEquals;

/**
 * The MIT License
 * <p>
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * <p>
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * <p>
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 * <p>
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * <p>
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 * <p>
 * Created by MacAni on 26/06/2017 11.37.
 */

@RunWith(MockitoJUnitRunner.class)
public class OrganDonorDeceasedCleanupSupplierTest {

    @Mock
    CleanupProperties cleanupProperties;
    @Mock
    CleanupService cleanupService;
    @Mock
    PersonInformationService personInformationService;
    @Mock
    DateTimeProvider dateTimeProvider;

    CleanupOperationsFactory cleanupOperationsFactory;

    private OrganDonorDeceasedCleanupSupplier organDonorDeceasedCleanupSupplier;

    @Before
    public void setUp() {
        cleanupOperationsFactory = Mockito.spy(new CleanupOperationsFactory(cleanupProperties, cleanupService, personInformationService, dateTimeProvider));
        organDonorDeceasedCleanupSupplier = new OrganDonorDeceasedCleanupSupplier(cleanupOperationsFactory);
    }

    @Test
    public void testMinlog2CleanupSupplier() {
        List<StackOperation> stackOperations = organDonorDeceasedCleanupSupplier.run();
        assertEquals(366, stackOperations.size());
        Mockito.verify(cleanupOperationsFactory, Mockito.times(366)).createCPRPrefixDeceasedCleanupSupplier(Mockito.anyString());
    }
}
