/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.organdonor.cleanup.stack.emigrated;


import dk.sundhedsdatastyrelsen.organdonor.backgroundjob.stack.StackOperation;
import dk.sundhedsdatastyrelsen.organdonor.backgroundjob.util.DateTimeProvider;
import dk.sundhedsdatastyrelsen.organdonor.cleanup.config.CleanupProperties;
import dk.sundhedsdatastyrelsen.organdonor.cleanup.factory.CleanupOperationsFactory;
import dk.sundhedsdatastyrelsen.organdonor.cleanup.service.CleanupException;
import dk.sundhedsdatastyrelsen.organdonor.service.PersonInformationService;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.junit.MockitoJUnitRunner;

import java.text.MessageFormat;
import java.time.LocalDate;
import java.time.ZonedDateTime;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * The MIT License
 * <p>
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * <p>
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * <p>
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 * <p>
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * <p>
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 * <p>
 * Created by MacAni on 26/06/2017 11.37.
 */

@RunWith(MockitoJUnitRunner.class)
public class CPRBatchEmigratedCleanupSupplierTest {
    @Mock
    CleanupOperationsFactory cleanupOperationsFactory;
    @Mock
    CleanupProperties cleanupProperties;
    @Mock
    PersonInformationService personInformationService;
    @Mock
    DateTimeProvider dateTimeProvider;

    //public List<String> cprList = Stream.of("1405123456", "1405234567", "1405345678", "1405456789", "1405567890").collect(Collectors.toList());
    //public List<String> filteredCprList = Stream.of("1405123456", "1405567890").collect(Collectors.toList());

    public String personIdType = "CPR";
    public CPRBatchEmigratedCleanupSupplier cprBatchEmigratedCleanupSupplier;

    @Before
    public void setUp() {
        Mockito.when(dateTimeProvider.now()).thenReturn(ZonedDateTime.now());
        Mockito.when(cleanupProperties.getPersoninformationEmigratedBatchSize()).thenReturn(10);
    }

    @Test
    public void testCPRBatchEmigratedCleanupSupplier5InCPRList(){
        List<String> cprList = buildCprList(5);
        List<String> filteredCprList = filterList(cprList, 2);

        Mockito.when(personInformationService.filterEmigrated(cprList, dateTimeProvider.now().toInstant(), "80")).thenReturn(filteredCprList);
        cprBatchEmigratedCleanupSupplier = new CPRBatchEmigratedCleanupSupplier(cleanupOperationsFactory, cleanupProperties, personInformationService, dateTimeProvider, "?", cprList);

        List<StackOperation> stackOperations = cprBatchEmigratedCleanupSupplier.run();
        Assert.assertEquals(1, stackOperations.size());
        Mockito.verify(personInformationService, Mockito.times(1)).filterEmigrated(cprList, dateTimeProvider.now().toInstant(), "80");
        Mockito.verify(cleanupOperationsFactory, Mockito.times(1)).createCPRCleanupEmigratedOperation("?", filteredCprList);
    }

    @Test
    public void testCPRBatchEmigratedCleanupSupplier100InCPRList(){
        List<String> cprList = buildCprList(100);
        List<String> filteredCprList = filterList(cprList, 2);
        Mockito.when(personInformationService.filterEmigrated(cprList.subList(0,10), dateTimeProvider.now().toInstant(), "80")).thenReturn(filteredCprList);

        cprBatchEmigratedCleanupSupplier = new CPRBatchEmigratedCleanupSupplier(cleanupOperationsFactory, cleanupProperties, personInformationService, dateTimeProvider, "?", cprList);
        List<StackOperation> stackOperations = cprBatchEmigratedCleanupSupplier.run();

        Assert.assertEquals(1, stackOperations.size());
        Mockito.verify(personInformationService, Mockito.times(10)).filterEmigrated(Mockito.anyList(), Mockito.any(), Mockito.eq("80"));
        Mockito.verify(cleanupOperationsFactory, Mockito.times(1)).createCPRCleanupEmigratedOperation("?", filteredCprList);
    }

    @Test
    public void testCPRBatchEmigratedCleanupSupplier0InCPRList(){
        List<String> cprList = buildCprList(0);
        List<String> filteredCprList = filterList(cprList, 0);

        cprBatchEmigratedCleanupSupplier = new CPRBatchEmigratedCleanupSupplier(cleanupOperationsFactory, cleanupProperties, personInformationService, dateTimeProvider, "?", cprList);
        List<StackOperation> stackOperations = cprBatchEmigratedCleanupSupplier.run();

        Assert.assertEquals(0, stackOperations.size());
        Mockito.verify(personInformationService, Mockito.times(0)).filterEmigrated(Mockito.anyList(), Mockito.any(), Mockito.eq("80"));
        Mockito.verify(cleanupOperationsFactory, Mockito.times(0)).createCPRCleanupEmigratedOperation("?", filteredCprList);
    }

    @Test
    public void testCPRBatchEmigratedCleanupSupplier100InCPRList0InFilteredList(){
        List<String> cprList = buildCprList(100);
        List<String> filteredCprList = filterList(cprList, 0);
        Mockito.when(personInformationService.filterEmigrated(cprList.subList(0,10), dateTimeProvider.now().toInstant(), "80")).thenReturn(filteredCprList);

        cprBatchEmigratedCleanupSupplier = new CPRBatchEmigratedCleanupSupplier(cleanupOperationsFactory, cleanupProperties, personInformationService, dateTimeProvider, "?", cprList);
        List<StackOperation> stackOperations = cprBatchEmigratedCleanupSupplier.run();

        Assert.assertEquals(0, stackOperations.size());
        Mockito.verify(personInformationService, Mockito.times(10)).filterEmigrated(Mockito.anyList(), Mockito.any(), Mockito.eq("80"));
        Mockito.verify(cleanupOperationsFactory, Mockito.times(0)).createCPRCleanupEmigratedOperation("?", filteredCprList);
    }

    @Test
    public void testCPRBatchEmigratedCleanupSupplierFailsFilteringEmigrated(){
        List<String> cprList = buildCprList(5);
        List<String> filteredCprList = filterList(cprList, 1);
        Mockito.when(personInformationService.filterEmigrated(cprList, dateTimeProvider.now().toInstant(), "80")).thenThrow(new RuntimeException("Test fejl"));

        cprBatchEmigratedCleanupSupplier = new CPRBatchEmigratedCleanupSupplier(cleanupOperationsFactory, cleanupProperties, personInformationService, dateTimeProvider, "?", cprList);
        CleanupException e = Assert.assertThrows(CleanupException.class, cprBatchEmigratedCleanupSupplier::run);
        Assert.assertTrue(e.getMessage(), e.getMessage().startsWith("Der opstod en fejl efter kald til PersonInformation for udrejste med skæringsdato:"));
        Assert.assertEquals("Test fejl", e.getCause().getMessage());
    }

    static List<String> buildCprList(int size){
        final MessageFormat messageFormat = new MessageFormat("{0,number,00}{1,number,00}{2,number,00}{3,number,0000}");
        final LocalDate base = LocalDate.of(1967, 4,17);
        return Stream.iterate(0, i -> i + 1)
                .limit(size)
                .map(i -> base.plusYears(i).plusMonths(i).plusDays(i))
                .map(d -> messageFormat.format(toArray(d.getDayOfMonth(), d.getMonthValue(), d.getYear() % 100, d.getYear())))
                .collect(Collectors.toList());
    }

    static List<String> filterList(List<String> in, int size){
        return in.stream().limit(size).collect(Collectors.toList());
    }

    static Object[] toArray(Object... values){
        return values;
    }
}
