/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.organdonor.cleanup.stack.emigrated;

import dk.sundhedsdatastyrelsen.organdonor.backgroundjob.stack.StackOperation;
import dk.sundhedsdatastyrelsen.organdonor.backgroundjob.util.DateTimeProvider;
import dk.sundhedsdatastyrelsen.organdonor.cleanup.config.CleanupProperties;
import dk.sundhedsdatastyrelsen.organdonor.cleanup.service.CleanupException;
import dk.sundhedsdatastyrelsen.organdonor.cleanup.service.CleanupService;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.junit.MockitoJUnitRunner;

import java.time.ZonedDateTime;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * The MIT License
 * <p>
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * <p>
 * Copyright (C) 2011 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 * <p>
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 * <p>
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * <p>
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 * <p>
 * Created by MacAni on 26/06/2017 11.37.
 */

@RunWith(MockitoJUnitRunner.class)
public class CPRCleanupMigratedOperationTest {

    @Mock
    CleanupProperties cleanupProperties;
    @Mock
    CleanupService cleanupService;
    @Mock
    DateTimeProvider dateTimeProvider;

    public List<String> personIdsForUpdate = Stream.of("1405123456", "1405234567", "1405345678", "1405456789", "1405567890").collect(Collectors.toList());
    public String personIdType = "CPR";

    CPREmigratedCleanupOperation cprEmigratedCleanupOperation;

    @Before
    public void setUp() {
        Mockito.when(dateTimeProvider.now()).thenReturn(ZonedDateTime.now());
        cprEmigratedCleanupOperation = new CPREmigratedCleanupOperation(cleanupService, dateTimeProvider, "?", personIdsForUpdate);
    }

    @Test
    public void testCPRCleanupOperation() {
        List<StackOperation> run = cprEmigratedCleanupOperation.run();
        Assert.assertTrue(run.isEmpty());
        Mockito.verify(cleanupService, Mockito.times(1)).updateEntriesByPersonIdBatch(personIdsForUpdate, dateTimeProvider.now().toInstant());
    }

    @Test
    public void testCPRCleanupOperationHandlingCleanupException() {
        Mockito.when(cleanupService.updateEntriesByPersonIdBatch(personIdsForUpdate, dateTimeProvider.now().toInstant())).thenThrow(new CleanupException("Error", new NullPointerException()));

        CleanupException cleanupException = Assert.assertThrows(CleanupException.class, cprEmigratedCleanupOperation::run);
        Assert.assertEquals("Error", cleanupException.getMessage());
        Assert.assertEquals(NullPointerException.class, cleanupException.getCause().getClass());


    }

    @Test
    public void testCPRCleanupOperationHandlingOtherException() {
        Mockito.when(cleanupService.updateEntriesByPersonIdBatch(personIdsForUpdate, dateTimeProvider.now().toInstant())).thenThrow(new RuntimeException("Error", new NullPointerException()));
        CleanupException cleanupException = Assert.assertThrows(CleanupException.class, cprEmigratedCleanupOperation::run);
        Assert.assertEquals("Der opstod en fejl under sletning af data: 5", cleanupException.getMessage());
        Assert.assertEquals(RuntimeException.class, cleanupException.getCause().getClass());
        Assert.assertEquals(NullPointerException.class, cleanupException.getCause().getCause().getClass());
    }

}
