/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.organdonor.personinformation;

import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringRunner;

import java.time.Instant;
import java.util.ArrayList;
import java.util.List;

@RunWith(SpringRunner.class)
@ContextConfiguration(classes = {PersonInformationTestConfig.class})
public class PersonInformationClientIT {
    @Autowired
    PersonInformationClient client;

    @Test
    public void testFilterByStatusCode01() {
        List<String> cprs = new ArrayList<>();
        cprs.add("0505050009");
        cprs.add("0505050010");
        cprs.add("0505050011");

        List<String> filteredCprs = client.doFilterByStatusCode(cprs, Instant.now(), "01");
        Assert.assertArrayEquals(new String[]{"0505050009", "0505050011"}, filteredCprs.toArray());

        Assert.assertTrue(filteredCprs.stream().map(client::getStatus).allMatch("01"::equals));
    }

    @Test
    public void testFilterByStatusCode80() {
        List<String> cprs = new ArrayList<>();
        cprs.add("0505054444");
        cprs.add("0505055555");
        cprs.add("0505056666");

        List<String> filteredCprs = client.doFilterByStatusCode(cprs, Instant.now(), "80");
        Assert.assertArrayEquals(new String[]{"0505055555"}, filteredCprs.toArray());

        Assert.assertTrue(filteredCprs.stream().map(client::getStatus).allMatch("80"::equals));
    }

    @Test
    public void testFilterByStatusCodeLargeInputLargeResponse() {
        List<String> cprs = new ArrayList<>();
        for (int i = 0; i < 10000; i++) {
            String cpr = "050505" + String.format("%04d", i);
            cprs.add(cpr);
        }

        List<String> filteredCprs = client.doFilterByStatusCode(cprs, Instant.now(), "01");
        Assert.assertEquals(27, filteredCprs.size());
    }

    @Test
    public void testFilterByStatusCodeLargeInputSmallResponse() {
        List<String> cprs = new ArrayList<>();
        for (int i = 0; i < 10000; i++) {
            String cpr = "050505" + String.format("%04d", i);
            cprs.add(cpr);
        }

        List<String> filteredCprs = client.doFilterByStatusCode(cprs, Instant.now(), "60");
        Assert.assertEquals(1, filteredCprs.size());
    }

    @Test
    public void testFilterByStatusCodeEmptyInput() {
        List<String> cprs = new ArrayList<>();
        List<String> filteredCprs = client.doFilterByStatusCode(cprs, Instant.now(), "01");
        Assert.assertEquals(0, filteredCprs.size());
    }

    @Test
    public void testFilterByStatusCodeNoInput() {
        List<String> cprs = null;
        List<String> filteredCprs = client.doFilterByStatusCode(cprs, Instant.now(), "01");
        Assert.assertEquals(0, filteredCprs.size());
    }
}
