/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2017 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.organdonor.wildfly;

import org.junit.Assert;
import org.junit.Test;

public class OrganDonorCleanupIT extends AbstractServletCallingIT{

    private static final String ODR_OPERATIONS_ADDRESS = System.getProperty("itOdrOperationsAddress", "http://localhost:8081/odr-operations");
    private static final String ODR_CLEANUP_DECEASED_ENDPOINT = ODR_OPERATIONS_ADDRESS + "/odr-slettejob/deceased/start";
    private static final String ODR_CLEANUP_EMIGRATED_ENDPOINT = ODR_OPERATIONS_ADDRESS + "/odr-slettejob/emigrated/start";
    private static final String ODR_CLEANUP_HEALTH_ENDPOINT = ODR_OPERATIONS_ADDRESS + "/odr-slettejob/health";
    private static final String ODR_CLEANUP_STATUS_ENDPOINT = ODR_OPERATIONS_ADDRESS + "/odr-slettejob/status";
    private static final String ODR_CLEANUP_ALARM_ENDPOINT = ODR_OPERATIONS_ADDRESS + "/odr-slettejob/alarm";

    @Test
    public void testDeceasedStartEndpoint() {
        System.out.println("ENDPOINT=" + ODR_CLEANUP_DECEASED_ENDPOINT);
        callAndVerify(ODR_CLEANUP_DECEASED_ENDPOINT, (responseCode, responseContent) -> {
            Assert.assertEquals("OK", responseContent);
            Assert.assertEquals(Integer.valueOf(200), responseCode);
        });
    }

    @Test
    public void testEmigratedStartEndpoint() {
        System.out.println("ENDPOINT=" + ODR_CLEANUP_EMIGRATED_ENDPOINT);
        callAndVerify(ODR_CLEANUP_EMIGRATED_ENDPOINT, (responseCode, responseContent) -> {
            Assert.assertEquals("OK", responseContent);
            Assert.assertEquals(Integer.valueOf(200), responseCode);
        });
    }

    @Test
    public void testHealthEndpoint() {
        System.out.println("ENDPOINT=" + ODR_CLEANUP_HEALTH_ENDPOINT);
        callAndVerify(ODR_CLEANUP_HEALTH_ENDPOINT, (responseCode, responseContent) -> {
            Assert.assertEquals("{{\"Person information service integration\":\"OK\"},{\"Database\":\"OK\"}}", responseContent);
            Assert.assertEquals(Integer.valueOf(200), responseCode);
        });
    }

    @Test
    public void testStatusEndpoint() {
        System.out.println("ENDPOINT=" + ODR_CLEANUP_STATUS_ENDPOINT);
        callAndVerify(ODR_CLEANUP_STATUS_ENDPOINT, (responseCode, responseContent) -> {
            Assert.assertEquals("Database: OK", responseContent);
            Assert.assertEquals(Integer.valueOf(200), responseCode);
        });
    }

    @Test
    public void testAlarmEndpoint() {
        System.out.println("ENDPOINT=" + ODR_CLEANUP_ALARM_ENDPOINT);
        callAndVerify(ODR_CLEANUP_ALARM_ENDPOINT, (responseCode, responseContent) -> {
            Assert.assertEquals("", responseContent);
            Assert.assertEquals(Integer.valueOf(200), responseCode);
        });
    }
}
